#
#    forest/command.py implements a de/serialization layer for commands
#    Copyright (C) 2002  Michael Urman
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
command.py

Implements a command layer for the forest message system.
Commands are as follows:

    get_timestamp(tree) -> unix modification time of tree
    get_timestamp(tree, branch) -> unix modification time of tree:branch
    get_info(tree) -> tree info structure (see below)



    tree info structure:
        pretty name
        director message
"""

def quote(string):
    """quote(string) -> string

    Return a version of the string with ticks around it, and internal ticks and
    backslashes escaped with backslashes"""

    result = "\\\\".join(string.split("\\"))
    result = r"\'".join(result.split("'")
    return "'%s'" % result

def dequote(string):
    """dequote(string) -> string

    Return the original version of a quote()d string, removing the surrounding
    ticks and debackslashing"""

    result = string[1:-1]
    result = "'".join(result.split(r"\'"))
    result = "\\".join(result.split("\\\\'"))
    return result

class command:

    def __init__(self, type, *args):
        """__init__(self, type, *args) -> command

        Initialize a command of a type and with arguments as given"""

        if type == 'parse':
            raise NotImplementedError
        else:
            self.type = type
            self.args = args

    def __str__(self):
        """__str__(self) -> string

        Return a string representation of the command"""

        return '%s(%s)' % (self.type, ', '.join(self.args))

    def __repr__(self):
        """__repr__(self) -> string

        Return a network-safe string representation of the command"""

        type = quote(self.type)
        args = map(quote, self.args)
        return '<commmand %s %x>' % (type, id(self))
